/*
 *  Copyright (C) 2006-2010 Leandro Nini
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef SID_H
#define SID_H

#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif

#if defined (HAVE_SIDPLAY_BUILDERS_RESID_H) || defined (HAVE_SIDPLAY_BUILDERS_HARDSID_H)

#include <sidplay/sidplay2.h>

#define HAVE_SIDPLAY2

#ifdef HAVE_SIDUTILS
# include <sidplay/utils/SidDatabase.h>
#endif

#include <fx.h>
#include "inputPlugin.h"
#include "stil.h"

/*****************************************************************/

typedef struct {
	FXString	engine;
	FXint		samplerate;
	FXint		channels;
#ifndef ALAN_SIDPLAY
	sample_t	precision;
	FXint		optimisation;
#endif
	sid2_env_t	environment;
	sid2_clock_t	clock;
	FXbool		forceClock;
	sid2_model_t	model;
	FXbool		filter;
	FXString	hvscPath;
	FXbool		psid_filter;
#ifdef HAVE_SIDUTILS
	FXString	filterIni;
#endif
} sidConfig_t;

/*****************************************************************/

#include "configFrame.h"

class sidConfig : public configFrame {
	FXDECLARE(sidConfig)

private:
#ifndef ALAN_SIDPLAY
	FXDataTarget		_optimisation;
#endif
	FXDataTarget		_filter;
	FXDataTarget		_hvscDir;
#ifdef HAVE_SIDUTILS
	FXDataTarget		_filterIni;
#endif

private:
	sidConfig() {};
	sidConfig(const sidConfig&);
	sidConfig& operator=(const sidConfig&);

public:
	sidConfig(FXPacker* win);
	virtual ~sidConfig() {};

public:
	enum{
		ID_FREQUENCY=FXDialogBox::ID_LAST,
		ID_CHANNELS,
#ifndef ALAN_SIDPLAY
		ID_BITS,
#endif
		ID_ENGINE,
		ID_ENVIRONMENT,
		ID_HVSC,
		ID_FILTER,
		ID_LAST,
	};

public:
	long onCmdFrequency(FXObject*, FXSelector, void*);
	long onCmdChannels(FXObject*, FXSelector, void*);
#ifndef ALAN_SIDPLAY
	long onCmdBits(FXObject*, FXSelector, void*);
#endif
	long onCmdEngine(FXObject*, FXSelector, void*);
	long onCmdEnvironment(FXObject*, FXSelector, void*);
	long onCmdHvsc(FXObject*, FXSelector, void*);
	long onCmdFilter(FXObject*, FXSelector, void*);
};

/*****************************************************************/

class sid : public inputPlugin {

	friend class sidConfig;

private:
	static sidConfig_t	_settings;
	static const char	_engines[2][8];

private:
	sidplay2	*_sidplay2;

	STIL		*_stil;

#ifdef HAVE_SIDUTILS
	SidDatabase	*_db;
	FXuint		_length;
	char		_md5[SIDTUNE_MD5_LENGTH+1];

	SidTuneMod	*_tune;
#else
	SidTune		*_tune;
#endif

private:
	void loadTune(const int num);

	void openHvsc(const FXString& hvscPath);

public:
	sid();
	~sid();

	static const char name[];

	/// Factory method
	static input* factory() { return new sid(); }

	/// Check if we support ext
	static bool supports(const char* fileName);

	/// Get supported extension
	FXString ext() const;

	/// Open file
	bool open(const FXString& fileName);

	/// Close file
	void close();

	/// Rewind to start
	const bool rewind();

	/// Number of subtunes
	unsigned int subtunes() const { return _tune?_tune->getInfo().songs:0; }

	/// Current subtune
	unsigned int subtune() const { return _tune?_tune->getInfo().currentSong:0; }

	/// Change subtune
	bool subtune(const unsigned int i);

	/// Get samplerate
	const unsigned int samplerate() const { return _tune?_settings.samplerate:0; }

	/// Get channels
	const unsigned int channels() const { return _tune?_settings.channels:0; }

	/// Get precision
	const sample_t precision() const {
#ifndef ALAN_SIDPLAY
		return _settings.precision;
#else
		return SAMPLE_FLOAT;
#endif
	}

	/// Callback function
	size_t fillBuffer(void* buffer, const size_t bufferSize, const unsigned int seconds);

	/// Open config dialog
	void config(FXPacker* win) { new sidConfig(win); }

	/// Get Music directory
	const FXString getMusicDir(void) { return _settings.hvscPath; }

	void loadSettings();

	void saveSettings();
};

#endif

#endif
